unit UColorConv;

interface

type TColorType = (ctUnknown, ctRGB24, ctYUY2, ctUYVY, ctYVYU);

procedure CalculateRGB(pSource, pTarget: PChar;
  const SizeSource, SizeTarget: Integer; ColorType: TColorType);

implementation uses Windows;

type
  TRGBQuad = packed record
    B, G, R: Byte;
  end;
  PRGBQuad = ^TRGBQuad;

  TYUY2 = packed record
    Y0, U0, Y1, V1: byte;
  end;
  PYUY2 = ^TYUY2;

  TUYVY = packed record
    U0, Y0, V1, Y1: Byte;
  end;
  PUYVY = ^TUYVY;

  TYVYU = packed record
    Y0, V1, Y1, U0: Byte;
  end;
  PYVYU = ^TYVYU;


procedure CalculateRGB(pSource, pTarget: PChar;
  const SizeSource, SizeTarget: Integer; ColorType: TColorType);

var
  pEndSource: PChar;
  pEndTarget: PChar;
  y0, y1, u0, v1: Integer;
  size: Word;
  c1, c2, c3: Integer;

  function CLIP(b: Integer): Byte; assembler
  begin
    b := b div 256;
    if b < 0 then b := 0;
    if b > 255 then b := 255;
    result := b;
  end;

begin
  pEndTarget := pTarget + SizeTarget;
  pEndSource := pSource + SizeSource;
  y0 := 0; y1 := 0; u0 := 0; v1 := 0;
  case ColorType of
    ctYUY2: size := 4;
    ctUYVY: size := 4;
  else exit;
  end;
  while (pSource < pEndSource) and (pTarget < pEndTarget) do
  begin
    case ColorType of
      ctYUY2:
        begin
          y0 := PYUY2(pSource)^.Y0;
          y1 := PYUY2(pSource)^.Y1;
          u0 := PYUY2(pSource)^.U0;
          v1 := PYUY2(pSource)^.V1;
        end;
      ctUYVY:
        begin
          y0 := PUYVY(pSource)^.Y0;
          y1 := PUYVY(pSource)^.Y1;
          u0 := PUYVY(pSource)^.U0;
          v1 := PUYVY(pSource)^.V1;
        end;
      ctYVYU:
        begin
          y0 := PYVYU(pSource)^.Y0;
          y1 := PYVYU(pSource)^.Y1;
          u0 := PYVYU(pSource)^.U0;
          v1 := PYVYU(pSource)^.V1;
        end;
    end;

    c1 := (y0 - 16) * 298;
    c2 := u0 - 128;
    c3 := v1 - 128;

    PRGBQuad(pTarget).r := Clip(c1 + 409 * c3 + 128);
    PRGBQuad(pTarget).g := Clip(c1 - 100 * c2 - 208 * c3 + 128);
    PRGBQuad(pTarget).b := Clip(c1 + 516 * c2 + 128);

    inc(pTarget, sizeof(TRGBQuad));

    c1 := (y1 - 16) * 298;

    PRGBQuad(pTarget).r := Clip(c1 + 409 * c3 + 128);
    PRGBQuad(pTarget).g := Clip(c1 - 100 * c2 - 208 * c3 + 128);
    PRGBQuad(pTarget).b := Clip(c1 + 516 * c2 + 128);

    inc(pTarget, sizeof(TRGBQuad));

    inc(pSource, size);
  end;
end;

end.
