unit UProperty;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, BaseClass, StdCtrls, ExtCtrls, XPMan, ComCtrls;

type
  TfrmPropPage = class(TFormPropertyPage)
    XPManifest1: TXPManifest;
    dlgOpen: TOpenDialog;
    GroupBox5: TGroupBox;
    lblHPos: TLabel;
    lblHStretch: TLabel;
    tbHorizontal: TTrackBar;
    tbHStretch: TTrackBar;
    GroupBox4: TGroupBox;
    lblVPos: TLabel;
    lblVStretch: TLabel;
    tbVertical: TTrackBar;
    tbVStretch: TTrackBar;
    GroupBox3: TGroupBox;
    tbPanScan: TTrackBar;
    rgAspect: TRadioGroup;
    gbShader: TGroupBox;
    lblFragment: TLabel;
    edFragment: TEdit;
    btnLoadFragment: TButton;
    btnLoadVertex: TButton;
    edVertex: TEdit;
    lblVertex: TLabel;
    btnCompile: TButton;
    Image1: TImage;
    procedure FormDestroy(Sender: TObject);
    procedure btnLoadFragmentClick(Sender: TObject);
    procedure btnLoadVertexClick(Sender: TObject);
    procedure tbHorizontalChange(Sender: TObject);
    procedure tbHStretchChange(Sender: TObject);
    procedure tbVerticalChange(Sender: TObject);
    procedure tbVStretchChange(Sender: TObject);
    procedure tbPanScanChange(Sender: TObject);
    procedure rgAspectClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    procedure ClosePropertyWindow;
  published
    function OnConnect(Unknown: IUnknown): HRESULT; override;
    function OnDisconnect: HRESULT; override;
    function OnApplyChanges: HResult; override;
  end;

var
  frmPropPage: TfrmPropPage;

implementation uses VidRenderer, formRenderer;

{$R *.dfm}

procedure TfrmPropPage.ClosePropertyWindow;
begin
  PostMessage(GetParent(GetParent(Handle)), WM_COMMAND, IDCANCEL, 0);
end;

function TfrmPropPage.OnConnect(Unknown: IUnknown): HRESULT;
begin
  if Unknown = nil then
    result := E_POINTER
  else
    if frmPropPage <> nil then
      result := E_FAIL
    else begin
      frmPropPage := Self;
      result := S_OK;
    end;
end;

function TfrmPropPage.OnDisconnect: HRESULT;
begin
  result := NOERROR;
end;

function TfrmPropPage.OnApplyChanges: HResult;
begin
  result := S_OK;
end;

procedure TfrmPropPage.FormDestroy(Sender: TObject);
begin
  frmPropPage := nil;
end;

procedure TfrmPropPage.btnLoadFragmentClick(Sender: TObject);
begin
  dlgOpen.Filter := 'Fragment Shader (*.frag)|*.frag';
  if dlgOpen.execute then edFragment.text := dlgOpen.FileName;
end;

procedure TfrmPropPage.btnLoadVertexClick(Sender: TObject);
begin
  dlgOpen.Filter := 'Vertex Shader (*.vert)|*.vert';
  if dlgOpen.execute then edVertex.text := dlgOpen.FileName;
end;

procedure TfrmPropPage.tbHorizontalChange(Sender: TObject);
begin
  frmRenderer.LeftOffset := tbHorizontal.Position;
end;

procedure TfrmPropPage.tbHStretchChange(Sender: TObject);
begin
  frmRenderer.HStretch := tbHStretch.Position / 10;
end;

procedure TfrmPropPage.tbVerticalChange(Sender: TObject);
begin
  frmRenderer.TopOffset := tbVertical.Position;
end;

procedure TfrmPropPage.tbVStretchChange(Sender: TObject);
begin
  frmRenderer.VStretch := -tbVStretch.Position / 10;
end;

procedure TfrmPropPage.tbPanScanChange(Sender: TObject);
begin
  frmRenderer.PanScan := (tbPanScan.Position / 100) - 1;
end;

procedure TfrmPropPage.rgAspectClick(Sender: TObject);
begin
  case rgAspect.ItemIndex of
    0: frmRenderer.AspectRatio := 0;
    1: frmRenderer.AspectRatio := 4 / 3;
    2: frmRenderer.AspectRatio := 14 / 9;
    3: frmRenderer.AspectRatio := 16 / 9;
  end;
end;

procedure TfrmPropPage.FormShow(Sender: TObject);
begin
  tbHorizontal.Position := frmRenderer.LeftOffset;
  tbHStretch.Position := round(frmRenderer.HStretch * 10);
  tbVertical.Position := frmRenderer.TopOffset;
  tbVStretch.Position := -round(frmRenderer.VStretch * 10);
  tbPanScan.Position := round((frmRenderer.PanScan * 100) - 1);
  if frmRenderer.AspectRatio = 4 / 3 then rgAspect.ItemIndex := 1 else
    if frmRenderer.AspectRatio = 14 / 9 then rgAspect.ItemIndex := 2 else
      if frmRenderer.AspectRatio = 16 / 9 then rgAspect.ItemIndex := 1 else
        rgAspect.ItemIndex := 0;
end;

initialization
  TBCClassFactory.CreatePropertyPage(TfrmPropPage, CLSID_PropertyPage);
end.

