{.$DEFINE DEBUG}

unit formRenderer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, BaseClass, DirectShow9, dglOpenGL, ExtCtrls, UColorConv;

type
  TfrmRenderer = class(TForm)
    tmrRenderer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure tmrRendererTimer(Sender: TObject);
  private
    FPSCount: Integer; // Counter for FPS
    FLastTickCount: DWord;

    FWidth: integer;
    FHeight: integer;
    FFormat: TVideoInfoHeader;
    FColorType: TColorType;
    FMessageDrain: hWnd;
    FDC: HDC;
    FRC: HGLRC;
    FTextureID: Cardinal;
    FUpdateTexture: Boolean;
    FVideoData: Pointer;
    FVideoSize: DWORD;
    FShader: DWord;
    FVideoArrived: Boolean;

    FAspectRatio: Double;
    FHStretch: Double;
    FVStretch: Double;
    FPanScan: Double;
    FLeftOffs: Integer;
    FTopOffs: Integer;
    FVideoRect: TRect;
    FDoubleBuffered: Boolean;

    procedure ResizeScene(Width, Height: Integer);
    procedure RenderScene;

    procedure UpdateVideoRect;

    procedure SetAspectRatio(const Value: Double);
    procedure SetPanScan(const Value: Double);
    procedure SetHStretch(const Value: Double);
    procedure SetVStretch(const Value: Double);
    procedure SetLeftOffset(const Value: Integer);
    procedure SetTopOffset(const Value: Integer);
  protected
    procedure WndProc(var Message: TMessage); override;
  public
    function DoSetWindowStyle(Style, WindowLong: LongInt): HRESULT;
    function DoGetWindowStyle(out Style: LongInt; WindowLong: LongInt): HRESULT;
    function DoShowWindow(ShowCmd: Longint): HResult;
    procedure DoPaintWindow(Erase: Boolean);
    procedure DoRenderSample(Sample: IMediaSample);
    procedure DoInitializeDirectDraw(Info: PVideoInfoHeader; SubTyp: TGuid);
    function AllowVideoType(SubType: TGuid): Boolean;
  published
    property MessageDrain: hWnd read FMessageDrain write FMessageDrain;

    property AspectRatio: Double read FAspectRatio write SetAspectRatio;
    property PanScan: Double read FPanScan write SetPanScan;
    property HStretch: Double read FHStretch write SetHStretch;
    property VStretch: Double read FVStretch write SetVStretch;
    property LeftOffset: Integer read FLeftOffs write SetLeftOffset;
    property TopOffset: Integer read FTopOffs write SetTopOffset;
  end;

var
  frmRenderer: TfrmRenderer;

implementation

{$R *.dfm}

function IsExtensionSupported(szTargetExtension: string): boolean;
var
  pszExtensions: string;
  pszWhere: integer;
begin
  pszWhere := Pos(' ', szTargetExtension);
  if (pszWhere <> 0) or (szTargetExtension = '') then
  begin
    Result := false;
    exit;
  end;
  pszExtensions := glGetString(GL_EXTENSIONS);
  if Pos(szTargetExtension, pszExtensions) = 0 then
  begin
    Result := false;
    exit;
  end;
  Result := true;
end;

function LoadShader(FFilename: string; VFilename: string = ''): GLHandleARB;
var
  ProgramObject, FragmentShaderObject, VertexShaderObject: GLHandleARB;
  FShader: TStringList;
  FShaderText: string;
  FShaderLength: Integer;
  VShader: TStringList;
  VShaderText: string;
  VShaderLength: Integer;
  useFragment: Boolean;
  useVertex: Boolean;
begin
  useFragment := FileExists(FFilename);
  useVertex := FileExists(VFilename);
  if (useFragment) or (useVertex) then
  begin
    ProgramObject := glCreateProgramObjectARB;

    if useFragment then
    begin
      FragmentShaderObject := glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
      FShader := TStringList.Create;
      FShader.LoadFromFile(FFilename);
      FShaderText := FShader.Text;
      FShaderLength := Length(FShaderText);
      glShaderSourceARB(FragmentShaderObject, 1, @FShaderText, @FShaderLength);
      glCompileShaderARB(FragmentShaderObject);
      glAttachObjectARB(ProgramObject, FragmentShaderObject);
      glDeleteObjectARB(FragmentShaderObject);
    end;

    if useVertex then
    begin
      VertexShaderObject := glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
      VShader := TStringList.Create;
      VShader.LoadFromFile(VFilename);
      VShaderText := VShader.Text;
      VShaderLength := Length(VShaderText);
      glShaderSourceARB(VertexShaderObject, 1, @VShaderText, @VShaderLength);
      glCompileShaderARB(VertexShaderObject);
      glAttachObjectARB(ProgramObject, VertexShaderObject);
      glDeleteObjectARB(VertexShaderObject);
    end;

    glLinkProgramARB(ProgramObject);

    Result := ProgramObject;
  end else
    Result := 0;
end;

function PossiblyEatMessage(hwndDrain: hWnd; Msg: Cardinal; wParam: WPARAM; lParam: LPARAM): Boolean; stdcall;
begin
  if ((hwndDrain <> 0) and not InSendMessage) then
  begin
    case Msg of
      WM_CHAR,
        WM_DEADCHAR,
        WM_KEYDOWN,
        WM_KEYUP,
        WM_LBUTTONDBLCLK,
        WM_LBUTTONDOWN,
        WM_LBUTTONUP,
        WM_MBUTTONDBLCLK,
        WM_MBUTTONDOWN,
        WM_MBUTTONUP,
        WM_MOUSEACTIVATE,
        WM_MOUSEMOVE,
      // If we pass this on we don't get any mouse clicks
      // WM_NCHITTEST,
      WM_NCLBUTTONDBLCLK,
        WM_NCLBUTTONDOWN,
        WM_NCLBUTTONUP,
        WM_NCMBUTTONDBLCLK,
        WM_NCMBUTTONDOWN,
        WM_NCMBUTTONUP,
        WM_NCMOUSEMOVE,
        WM_NCRBUTTONDBLCLK,
        WM_NCRBUTTONDOWN,
        WM_NCRBUTTONUP,
        WM_RBUTTONDBLCLK,
        WM_RBUTTONDOWN,
        WM_RBUTTONUP,
        WM_SYSCHAR,
        WM_SYSDEADCHAR,
        WM_SYSKEYDOWN,
        WM_SYSKEYUP:
        begin
{$IFDEF DEBUG}
          DbgLog('OpenGL Video Renderer: Forwarding Message $' + inttohex(Msg, 8) + ' drain');
{$ENDIF}
          PostMessage(hwndDrain, Msg, wParam, lParam);
          Result := True;
          Exit;
        end;
    end;
  end;
  Result := False;
end;

procedure TfrmRenderer.WndProc(var Message: TMessage);
begin
  if PossiblyEatMessage(FMessageDrain, Message.Msg, Message.WParam, Message.LParam) then Exit;
  case Message.Msg of
    WM_ERASEBKGND:
      begin
        Message.Result := 0;
        Exit;
      end;
  end;

  inherited WndProc(Message);
end;

procedure TfrmRenderer.SetPanScan(const Value: Double);
begin
  if FPanScan <> Value then
  begin
    FPanScan := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.SetAspectRatio(const Value: Double);
begin
  if (FAspectRatio <> Value) or (FAspectRatio = 0) then
  begin
    if (Value = 0) and (FHeight > 0) then
      FAspectRatio := FWidth / FHeight else
      FAspectRatio := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.SetHStretch(const Value: Double);
begin
  if FHStretch <> Value then
  begin
    FHStretch := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.SetVStretch(const Value: Double);
begin
  if FVStretch <> Value then
  begin
    FVStretch := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.SetLeftOffset(const Value: Integer);
begin
  if FLeftOffs <> Value then
  begin
    FLeftOffs := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.SetTopOffset(const Value: Integer);
begin
  if FTopOffs <> Value then
  begin
    FTopOffs := Value;
    UpdateVideoRect;
  end;
end;

procedure TfrmRenderer.UpdateVideoRect;
var
  H1, H2, W1, W2: Integer;
  R: TRect;
begin
  if FAspectRatio <> 0 then
  begin
    H1 := ClientHeight;
    W1 := ClientWidth;

    if H1 * W1 = 0 then exit;

    if (W1 / H1) > FAspectRatio then
    begin
      W2 := Round(H1 * FAspectRatio);
      R.Left := (W1 - W2) shr 1;
      R.Right := R.Left + W2;
      R.Top := 0;
      R.Bottom := H1;
    end
    else
    begin
      H2 := Round(W1 / FAspectRatio);
      R.Left := 0;
      R.Right := W1;
      R.Top := (H1 - H2) shr 1;
      R.Bottom := R.Top + H2;
    end;
  end
  else
  begin
    R.Left := 0;
    R.Top := Width;
    R.Top := 0;
    R.Bottom := height;
  end;

  W1 := Round(FHStretch * (R.Right - R.Left)) div 4;
  R.Left := R.Left - W1;
  R.Right := R.Right + W1;

  H1 := Round(FVStretch * (R.Bottom - R.Top)) div 4;
  R.Top := R.Top - H1;
  R.Bottom := R.Bottom + H1;

  H1 := Round(FPanScan * (R.Bottom - R.Top)) div 2;
  W1 := Round(FPanScan * (R.Right - R.Left)) div 2;

  R.Left := (R.Left - W1 + FLeftOffs);
  R.Right := R.Right + W1 + FLeftOffs;
  R.Top := (R.Top - H1 + FTopOffs);
  R.Bottom := R.Bottom + H1 + FTopOffs;

  FVideoRect := r;
end;

function TfrmRenderer.DoSetWindowStyle(Style, WindowLong: LongInt): HRESULT;
var
  WindowRect: TRect;
  WindowFlags: Cardinal;
begin

  // Set the new style flags for the window
  SetWindowLong(Handle, WindowLong, Style);
  WindowFlags := SWP_SHOWWINDOW or SWP_FRAMECHANGED or SWP_NOACTIVATE;
  WindowFlags := WindowFlags or SWP_NOZORDER or SWP_NOSIZE or SWP_NOMOVE;

  // Show the window again in the current position

  if Visible then
  begin
    SetWindowPos(Handle, // Base window handle
      HWND_TOP, // Just a place holder
      0, 0, 0, 0, // Leave size and position
      WindowFlags); // Just draw it again
    Result := NOERROR;
    Exit;
  end;

  // Move the window offscreen so the user doesn't see the changes

  MoveWindow(Handle, // Base window handle
    GetSystemMetrics(SM_CXSCREEN), // Current desktop width
    GetSystemMetrics(SM_CYSCREEN), // Likewise it's height
    Width, // Use the same width
    Height, // Keep height same to
    True); // May as well repaint

  // Now show the previously hidden window

  SetWindowPos(Handle, // Base window handle
    HWND_TOP, // Just a place holder
    0, 0, 0, 0, // Leave size and position
    WindowFlags); // Just draw it again

  ShowWindow(Handle, SW_HIDE);

  if (ParentWindow > 0) then MapWindowPoints(HWND_DESKTOP, ParentWindow, WindowRect, 2);

  MoveWindow(Handle, // Base window handle
    WindowRect.left, // Existing x coordinate
    WindowRect.top, // Existing y coordinate
    Width, // Use the same width
    Height, // Keep height same to
    True); // May as well repaint

  Result := NOERROR;
end;

function TfrmRenderer.DoGetWindowStyle(out Style: LongInt; WindowLong: LongInt): HRESULT;
begin
  Style := GetWindowLong(Handle, WindowLong);
  Result := NOERROR;
end;

function TfrmRenderer.DoShowWindow(ShowCmd: Longint): HResult;
begin
  ShowWindow(Handle, ShowCmd);
  Result := NOERROR;
end;

procedure TfrmRenderer.DoPaintWindow(Erase: Boolean);
begin
  InvalidateRect(Handle, nil, Erase);
end;

procedure TfrmRenderer.DoRenderSample(Sample: IMediaSample);
var
  Bits: PByte;
  Size: DWord;
begin
  Sample.GetPointer(Bits);
  Size := Sample.GetActualDataLength;
  case FColorType of
    ctUnknown, ctRGB24: begin
        if Size > FVideoSize then Size := FVideoSize;
        Move(Bits^, FVideoData^, Size);
      end;
  else begin
      if Size > FVideoSize then Size := FVideoSize;
      Move(Bits^, FVideoData^, Size);
      CalculateRGB(PChar(Bits), PChar(FVideoData), Size, FVideoSize, FColorType);
    end;
  end;
  FVideoArrived := True;
end;

function TfrmRenderer.AllowVideoType(SubType: TGuid): Boolean;
begin
  result := IsEqualGUID(subtype, MEDIASUBTYPE_RGB24) or
    IsEqualGUID(subtype, MEDIASUBTYPE_YUY2) or
    isEqualGUID(subtype, MEDIASUBTYPE_UYVY) or
    isEqualGUID(subtype, MEDIASUBTYPE_YVYU);
end;

procedure TfrmRenderer.DoInitializeDirectDraw(Info: PVideoInfoHeader; SubTyp: TGuid);
begin
  FFormat := Info^;
  FWidth := Info.bmiHeader.biWidth;
  FHeight := Info.bmiHeader.biHeight;
  if FVideoSize <> 0 then FreeMem(FVideoData, FVideoSize);
  FVideoSize := Abs(FWidth * FHeight * 3);
  Getmem(FVideoData, FVideoSize);

  if IsEqualGUID(Subtyp, MEDIASUBTYPE_RGB24) then FColorType := ctRGB24 else
    if IsEqualGUID(Subtyp, MEDIASUBTYPE_YUY2) then FColorType := ctYUY2 else
      if IsEqualGUID(Subtyp, MEDIASUBTYPE_UYVY) then FColorType := ctUYVY else
        if IsEqualGUID(Subtyp, MEDIASUBTYPE_YVYU) then FColorType := ctYVYU else
          FColorType := ctUnknown;

  FUpdateTexture := True;
  ClientWidth := FWidth;
  ClientHeight := FHeight;
end;

procedure TfrmRenderer.FormCreate(Sender: TObject);
var
  pfd: TPixelFormatDescriptor;
  p: Integer;
begin
  InitOpenGL;
  FDoubleBuffered := True;

  FAspectRatio := 4 / 3;
  FPSCount := 0;
  FLastTickCount := GetTickCount;

  FMessageDrain := 0;
  FVideoData := nil;
  FVideoSize := 0;

  FDC := getDC(Handle);

  fillchar(pfd, sizeof(pfd), 0);
  with pfd do
  begin
    nSize := SizeOf(TPIXELFORMATDESCRIPTOR);
    nVersion := 1;
    dwFlags := PFD_DRAW_TO_WINDOW or PFD_SUPPORT_OPENGL;
    if FDoubleBuffered then
      dwFlags := dwFlags or PFD_DOUBLEBUFFER;

    iPixelType := PFD_TYPE_RGBA;
    cColorBits := 32;
    cDepthBits := 16;
    iLayerType := PFD_MAIN_PLANE;
  end;
  p := choosePixelFormat(FDC, @pfd);

  SetPixelFormat(FDC, p, @pfd);
  FRC := wglCreateContext(FDC);
  wglMakeCurrent(FDC, FRC);
  FTextureID := 0;
  FUpdateTexture := False;
  FTextureID := 0;
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);

  {if IsExtensionSupported('GL_ARB_shader_objects') then
  begin
    FShader := LoadShader('simple_specular.frag', '');
    if FShader <> 0 then glUseProgramObjectARB(FShader);
  end;}
end;

procedure TfrmRenderer.FormDestroy(Sender: TObject);
begin
  if FVideoSize > 0 then
    FreeMem(FVideoData, FVideoSize);
  FVideoSize := 0;
  FVideoData := nil;

  wglMakeCurrent(0, 0);
  ReleaseDC(Handle, FDC);
  wglDeleteContext(FRC);
end;

procedure TfrmRenderer.ResizeScene(Width, Height: Integer);
begin
  if (Height = 0) then Height := 1;

  glViewport(0, 0, Width, Height); // Set the viewport for the OpenGL window
  glMatrixMode(GL_PROJECTION); // Change Matrix Mode to Projection
  glLoadIdentity(); // Reset View
  gluPerspective(45.0, Width / Height, 1.0, 100.0); // Do the perspective calculations. Last value = max clipping depth

  glMatrixMode(GL_MODELVIEW); // Return to the modelview matrix
  glLoadIdentity(); // Reset View

  UpdateVideoRect;
end;

procedure TfrmRenderer.RenderScene;
begin
  if FUpdateTexture then
  begin
    if FTextureID <> 0 then
      glDeletetextures(FTextureID, FVideoData);

    glEnable(GL_TEXTURE_RECTANGLE_ARB);

    glGenTextures(1, @FTextureID);
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB, FTextureID);

    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

    glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGB, FWidth, FHeight, 0,
      GL_BGR, GL_UNSIGNED_BYTE, nil);

// Update Shader
    glUniform1iARB(glGetUniformLocationARB(FShader, PGLCharARB('VideoTex')), FTextureID);
    FUpdateTexture := False;
  end;

  if FDoubleBuffered or FVideoArrived then
  begin
    glMatrixMode(GL_PROJECTION); // Change Matrix Mode to Projection
    glLoadIdentity(); // Reset View
    glOrtho(0, ClientWidth, 0, ClientHeight, 0, 100);

    glClearColor(0, 0, 0, 0);
    glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity;
  end;

  if FVideoArrived then
  begin
    FVideoArrived := False;
    glEnable(GL_TEXTURE_RECTANGLE_ARB);
    glTexSubImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, 0, 0, FWidth, FHeight,
      GL_BGR, GL_UNSIGNED_BYTE, FVideoData);
    glBindTexture(GL_TEXTURE_RECTANGLE_ARB, FTextureID);
  end else
    if not FDoubleBuffered then exit;

  glColor3f(1, 1, 1);

  glBegin(GL_QUADS);
  if FColorType in [ctRGB24, ctUnknown] then
  begin
    glTexCoord2f(0, 0); glVertex3f(FVideoRect.Left, FVideoRect.Top, 0.0);
    glTexCoord2f(FWidth, 0); glVertex3f(FVideoRect.Right, FVideoRect.Top, 0.0);
    glTexCoord2f(FWidth, FHeight); glVertex3f(FVideoRect.Right, FVideoRect.Bottom, 0.0);
    glTexCoord2f(0, FHeight); glVertex3f(FVideoRect.Left, FVideoRect.Bottom, 0.0);
  end else
  begin
    glTexCoord2f(0, FHeight); glVertex3f(FVideoRect.Left, FVideoRect.Top, 0.0);
    glTexCoord2f(FWidth, FHeight); glVertex3f(FVideoRect.Right, FVideoRect.Top, 0.0);
    glTexCoord2f(FWidth, 0); glVertex3f(FVideoRect.Right, FVideoRect.Bottom, 0.0);
    glTexCoord2f(0, 0); glVertex3f(FVideoRect.Left, FVideoRect.Bottom, 0.0);
  end;
  glEnd();

  if FDoubleBuffered then SwapBuffers(FDC);

  if GetTickCount - FLastTickCount >= 1000 then
  begin
    caption := format('FPS: %2d', [FPSCount]);
    FPSCount := 0;
    FLastTickCount := GetTickCount;
  end;
  Inc(FPSCount);
end;

procedure TfrmRenderer.FormResize(Sender: TObject);
begin
  ResizeScene(ClientWidth, ClientHeight);
  FVideoArrived := True;
end;

procedure TfrmRenderer.tmrRendererTimer(Sender: TObject);
begin
  RenderScene;
end;

end.

